@@ -16,7 +16,7 @@ import tkinter
 import tkinter.ttk as ttk
 from tkmacosx.utils import (SYSTEM_DEFAULT, STDOUT_WARNING,
                             _cnfmerge, _bind, _Canvas, check_param,
-                            _info_button, _on_press_color, 
+                            _info_button, _on_press_color, get_hex,
                             get_shade, check_function_equality)
 import tkmacosx.utils.colorvar_patches as cp
 
@@ -47,7 +47,7 @@ BUTTON_ITEMS = (
     '_txt', '_bit', '_img', '_bd_color', '_border', '_tf'
 )
 
-BORDER_INTENSITY = 0.06
+BORDER_INTENSITY = 0.075
 
 _warning_msg_shown = [False]
 
@@ -167,13 +167,9 @@ class _button_properties:
             if self.cnf.get('bordercolor') == _get_master_bg():
                 self.cnf.pop('bordercolor', None)
                 self.cnf.pop('highlightbackground', None)
-            bd_color = self.cnf.get(
-                'bordercolor', get_shade(self['bg'], 0.04, 'auto-120'))
+            bd_color = self.cnf.get('bordercolor', SYSTEM_DEFAULT.bg)
             if bd_color == '':
-                bd_color = get_shade(self['bg'], 0.04, 'auto-120')
-            elif bd_color.lower() == 'default':
-                bd_color = get_shade('white', 0.04, 'auto-120')
-            self.cnf.update({'bordercolor': bd_color})
+                bd_color = SYSTEM_DEFAULT.bg
             self.cnf['highlightbackground'] = self.cnf['bordercolor'] = bd_color
             if self.itemcget('_bd_color', 'outline') != bd_color:
                 _opt[1] = [('itemconfigure', '_bd_color'), {'outline': bd_color}, None]
@@ -229,9 +225,7 @@ class _button_properties:
                     {'className': 'overforeground', 'sequence': '<Leave>',
                     'func': fn.get('<Leave>')}]
             if self._mouse_state_condition():
-                # [issue-3] doesn't change if overrelief is on.
-                # [issue-3] (FIXED) using after with 0ms delay
-                #           fixes the issue. To be safe delay is 1ms.
+    
                 self.after(1, lambda: _Canvas._configure(self,
                     ('itemconfigure', '_txt'), {'fill': kw.get('overforeground')}, None))
         elif kw.get('overforeground') == '':
@@ -408,7 +402,7 @@ class _button_items:
     
     def _bd_color(self, *ags, **kw):
         "Border color item."
-        bd_color = self.cnf.get('bordercolor', get_shade(self['bg'], 0.04, 'auto-120'))
+        bd_color = self.cnf.get('bordercolor', SYSTEM_DEFAULT.bg)
         if self._type == 'circle':
             pad = 2
             width = r = int(self.cnf.get('width', 87)/2)  # radius = x = y (in pixels)
@@ -429,7 +423,7 @@ class _button_items:
         bo_color = get_shade(self['bg'], BORDER_INTENSITY, 'auto-120')
         if self._type == 'circle':
             pad = 2
-            r = int(self.cnf.get('width', 87)/2)  # radius = x = y
+            r = int(self.cnf.get('width', 87)/2)  # radius = x = y (in pixels)
             _bo_points = (pad, pad, r*2-(pad+1), r*2-(pad+1))
             return self._create('oval', _bo_points, {
                 'tag': '_border', 'outline': self.cnf.get('bordercolor', bo_color) })
@@ -449,7 +443,7 @@ class _button_items:
         if self._type == 'circle':
             pad = 1
             width = self.cnf.get('focusthickness', 2)
-            r = int(self.cnf.get('width', 87)/2)  # radius = x = y
+            r = int(self.cnf.get('width', 87)/2)  # radius = x = y (in pixels)
             _tk_points = (pad+width, pad+width, r*2-width-pad, r*2-width-pad)
             return self._create('oval', _tk_points, {
                     'tag': '_tf', 'width': width,
@@ -498,9 +492,11 @@ class _button_functions:
         con2 = bool(self.winfo_containing(*self.winfo_pointerxy()) == self)
         return con1 and con2
     
-    def _make_dictionaries(self, cnf={}, kw={}):
+    def _make_dictionaries(self, cnf=None, kw=None):
         """Internal function.\n
         Merges kw into cnf and removes None values."""
+        cnf = cnf or {}
+        kw = kw or {}
         kw['bordercolor'] = kw['highlightbackground'] = kw.get(
             'bordercolor', kw.get('highlightbackground'))
         kw = {k: v for k, v in kw.items() if v is not None}
@@ -598,8 +594,6 @@ class _button_functions:
                            '<Leave>': if_state(lambda _: self._configure(
                                 'configure', {'_relief': self._rel[0]}))},
 
-            # [issue-3] doesn't change if overrelief is on.
-            # [issue-3] (FIXED) using after with 0ms delay fixes the issue. To be safe delay is 1ms.
             'overforeground': {'<Enter>': if_state(lambda _: self.after(1, overfg, 'enter')),
                                '<Leave>': if_state(lambda _: overfg('leave'))},
             
@@ -748,16 +742,14 @@ class _button_functions:
                 if self.focus_get() is None:
                     self._tmp_bg = self['bg']
                     # [BUG] winfo_rgb doesn't read mac system colors.
-                    color1 = self.winfo_rgb(self['highlightbackground'])
-                    color1 = [int((int(i/257) + 255)/2) for i in color1]
-                    color1 = '#%02x%02x%02x' % (color1[0], color1[1], color1[2])
+                    color1 = get_hex(self['highlightbackground'], self)
                     color2 = get_shade(color1, intensity, 'auto-120')
                     _Canvas._configure(self, 'configure', {'bg': color1}, None)
                     _Canvas._configure(self, ('itemconfigure', '_border'),
                             {'outline': color2}, None)
                 if self.focus_get():
                     if getattr(self, '_tmp_bg', False):
-                        _Canvas._configure(self, 'configure',{'bg': self._tmp_bg}, None)
+                        _Canvas._configure(self, 'configure', {'bg': self._tmp_bg}, None)                    
                     color = get_shade(self['bg'], intensity, 'auto-120')
                     _Canvas._configure(self, ('itemconfigure', '_border'),
                              {'outline': color}, None)
@@ -816,9 +808,7 @@ class _button_functions:
             cur_focus.focus()
         else:
             self.master.focus()
-            # [issue-7] Need a better fix to get focus back to the button if it has it previously
             if cur_focus == self:
-                # [issue-7] (fixed) using after with 1ms delay solves the issue.
                 self._after_IDs[3] = self.after(1, self.focus)
 
     def _active(self, value):
@@ -879,7 +869,7 @@ class _button_functions:
             if self.cnf.get('repeatdelay', 0) and self.cnf.get('repeatinterval', 0) and self._rpinloop:
                 self._rpin = self.after(self.cnf.get('repeatinterval', 0), cmd)
             if self.cnf.get('command'):
-                self.cnf['command']() 
+                self.cnf['command']()
 
         def on_enter(*a):
             """Internal function.\n
@@ -922,6 +912,23 @@ class _button_functions:
         _bind(self, {'className': 'on_press_enter', 'sequence': '<Enter>'},
               {'className': 'on_press_leave', 'sequence': '<Leave>'},
               {'className': 'button_command', 'sequence': '<ButtonRelease-1>'})
+    
+    def _on_spacebar_press(self, *ags):
+        """Internal function. Triggers when spacebar is pressed"""
+        _bind(self,
+            {'className': 'spacebar_tab_remove','sequence': '<Tab>', 'func': self._on_release}
+        )
+        self._active("on_press")
+        if self.cnf.get('command'):
+            self.cnf['command']()
+
+    def _on_spacebar_release(self, *ags):
+        """Internal function. Triggers when the spacebar is released"""
+        _bind(self,
+            {'className': 'spacebar_tab_remove','sequence': '<Tab>'}
+        )
+        if ags and ags[0].keysym == 'space':
+            self._on_release(*ags)
 
     def _compound(self, flag, width, height):
         """Internal function.\n
@@ -964,8 +971,8 @@ class _button_functions:
         item_height = bbox[3] - bbox[1]
         default_padx = 2 + int(item_width/2)
         default_pady = 0 + int(item_height/2)
-        width = self.cnf['width'] = self.winfo_width()
-        height = self.cnf['height'] = self.winfo_height()
+        width = self.cnf['width'] = self.cnf.get('width', self.winfo_width())
+        height = self.cnf['height'] = self.cnf.get('height', self.winfo_height())
 
         # Center
         x = width / 2
@@ -1008,13 +1015,13 @@ class ButtonBase(_Canvas, _button_functions):
         self._fixed_size = {'w': False, 'h': False}
         self._var_cb = None
         self.cnf = {}
-        
+
         cnf = {}
         for i in kw.copy().keys():
             if i in BUTTON_FEATURES:
                 cnf[i] = kw.pop(i, None)
 
-        cnf['fg'] = cnf['foreground'] = cnf.get('fg', cnf.get('foreground', SYSTEM_DEFAULT.fg))
+        cnf['fg'] = cnf['foreground'] = cnf.get('fg', cnf.get('foreground', 'black'))
         cnf['anchor'] = cnf.get('anchor', 'center')
         cnf['justify'] = cnf.get('justify', 'center')
         cnf['borderless'] = cnf.get('borderless', False)
@@ -1043,17 +1050,17 @@ class ButtonBase(_Canvas, _button_functions):
             ra = int(cnf['radius']*2 + 4)
             kw['width'] = kw['height'] = kw.get('width', kw.get('height', ra))
         else:
-            kw['width'] = kw.get('width', 87)
-            kw['height'] = kw.get('height', 24)
+            kw['width'] = cnf.get('width', 87)
+            kw['height'] = cnf.get('height', 24)
 
         kw['takefocus'] = kw.get('takefocus', 1)
-        kw['bg'] = kw.pop('bg', kw.pop('background', SYSTEM_DEFAULT.bg))
+        kw['bg'] = kw.pop('bg', kw.pop('background', 'white'))
         kw['highlightthickness'] = kw.get('highlightthickness', 0)
 
         _Canvas.__init__(self, master=master, **kw)
         self.cnf = check_param(self, 'button', **cnf)
         self.cnf['bordercolor'] = self.cnf['highlightbackground'] = self.cnf.get(
-            'bordercolor', self.cnf.get('highlightbackground', get_shade(self['bg'], 0.04, 'auto-120')))
+            'bordercolor', self.cnf.get('highlightbackground', SYSTEM_DEFAULT.bg))
 
         self._buttons.append(self)
         self._size = (self.winfo_width(), self.winfo_height())
@@ -1065,11 +1072,12 @@ class ButtonBase(_Canvas, _button_functions):
             self._rel = ('flat', False)
 
         _bind(self,
-              {'className': 'button_release',
-                  'sequence': '<ButtonRelease-1>', 'func': self._on_release},
-              {'className': 'button_press',
-                  'sequence': '<Button-1>', 'func': self._on_press},
-              {'className': 'set_size', 'sequence': '<Configure>', 'func': self._set_size})
+              {'className': 'button_release','sequence': '<ButtonRelease-1>', 'func': self._on_release},
+              {'className': 'button_press','sequence': '<Button-1>', 'func': self._on_press},
+              {'className': 'set_size', 'sequence': '<Configure>', 'func': self._set_size},
+              {'className': 'button_spacebar', 'sequence': '<space>', 'func': self._on_spacebar_press},
+              {'className': 'button_spacebar', 'sequence': '<KeyRelease>', 'func': self._on_spacebar_release}
+        )
 
         self._focus_in_out(BORDER_INTENSITY)
         self._configure1(self.cnf)
